# ------------------------------------------------------------------------------
# Effects of Occurrence Data Density on Conservation Prioritization Strategies
# Description: Script to run all analyses
# Authors: Marlon E. Cobos, Claudia Nuez-Penichet, Peter D. Campbell, 
# Jacob C. Cooper, Fernando Machado-Stredel, Narayani Barve, Uzma Ashraf, 
# Abdelghafar A. Alkishe, Eric Ng'eno, Rahul Raveendran Nair, P. Joser Atauchi, 
# Adeola Adeboje, A. Townsend Peterson
# Date updated: 28/12/2022 (dd/mm/yyyy)
# ------------------------------------------------------------------------------



# Description ------------------------------------------------------------------
# Data for the species used in our analyses was downloaded from eBird:
# https://ebird.org/home
#
# Our downloaded data contains records until June 2022 for the USA only. 
# A similar search and download can be done at:
# https://science.ebird.org/en/use-ebird-data/download-ebird-data-products
# In our case the file downloaded is "ebd_US_relJun-2022.txt.gz", a compressed 
# file of ~80 GB.
#
# We have provided the filtered version as an RData file to facilitate sharing 
# reproducibility ("Data/ict_data_breeding.RData"). If this file is used, the 
# chunk of code "Read and filter data" can be skipped. 
#
# Layers to represent protected areas and human footprint were provided as part
# of the data package that contains this script.
#
# The software Zonation v4 is required to run analyses, manuals and the software
# are available at: https://www.syke.fi/en-US/Research__Development/Nature/Specialist_work/Zonation_in_Finland/Zonation_software
# Obtain Zonation from GitHub at: https://github.com/cbig/zonation-core/releases
#
# Notice that the a significant amount of results will be written in the working 
# directory.
# ------------------------------------------------------------------------------



# R packages ------------------------------------------------------------------
# packages from CRAN
library(prism)
library(terra)
library(raster)
library(zonator)
library(landscapemetrics)


# packages from GitHub
## install.packages("remotes")  # if needed
## remotes::install_github("fmachados/grinnell")
library(grinnell)

## the following packages require compilation tools
## if not installed, we recommend to update R and Rstudio 

## in Windows install Rtools 4.2 
## (https://cran.r-project.org/bin/windows/Rtools/rtools42/rtools.html)

## in MAC install xcode (try running in the terminal xcode-select --install. 
## More instructions: 
## https://www.freecodecamp.org/news/how-to-download-and-install-xcode/)

## in linux it will tell the compilation tools needed while installing
## remotes::install_github("marlonecobos/ellipsenm")
## remotes::install_github("marlonecobos/kuenm")
library(ellipsenm)
library(kuenm)

## to run Maxent from kuenm you JDK needs to be installed and Maxent should be 
## in a local directory
# ------------------------------------------------------------------------------



# Working directory ------------------------------------------------------------
setwd("Your/Directory")  # change this accordingly
# ------------------------------------------------------------------------------



# Read and filter data ---------------------------------------------------------
# read a small part of table to check columns
bdata <- read.table("Data/ebd_US_relJun-2022.txt.gz", sep = "\t", header = TRUE, 
                    fill = TRUE, nrows = 100)

## check columns
colnames(bdata)

# vector of columns to be read and others to be ignored (NULL are ignored)
cols <- c(rep("NULL", 6), NA, rep("NULL", 8), NA, rep("NULL", 12), NA, NA, NA,
          rep("NULL", 18))

# read all rows of the columns needed
bdata_all  <- read.table("Data/ebd_US_relJun-2022.txt.gz", sep = "\t", 
                         header = TRUE, fill = TRUE, colClasses = cols, 
                         stringsAsFactors = FALSE)


# filter data
## species to be kept
sp_vector <- c(
  "Xanthocephalus xanthocephalus", "Dolichonyx oryzivorus", "Sturnella magna", 
  "Sturnella neglecta", "Icterus spurius", "Icterus galbula", 
  "Icterus bullockii", "Icterus cucullatus", "Icterus gularis", 
  "Icterus parisorum", "Icterus graduacauda", "Agelaius phoeniceus", 
  "Agelaius tricolor", "Euphagus carolinus", "Euphagus cyanocephalus",
  "Molothrus aeneus", "Molothrus ater", "Quiscalus quiscula", "Quiscalus major", 
  "Quiscalus mexicanus"
)

## keep only the list of species
ict_data <- bdata_all[bdata_all$SCIENTIFIC.NAME %in% sp_vector, ]

## exclude NA
ict_data <- na.omit(ict_data)

## exclude empty cells
ict_data <- ict_data[ict_data$OBSERVATION.DATE != "", ]

## exclude duplicates
ict_data <- ict_data[!duplicated(paste(ict_data$SCIENTIFIC.NAME, 
                                       ict_data$LATITUDE,
                                       ict_data$LONGITUDE,
                                       ict_data$OBSERVATION.DATE)), ]

## keep records for after 2002
ict_data <- ict_data[as.Date(ict_data$OBSERVATION.DATE) >= 
                       as.Date("2002-01-01"), ]

ict_data <- na.omit(ict_data)

## keep only data for breeding periods
months <- format(as.Date(c("2002-05-01", "2002-06-01", "2002-07-01")), "%m")

ict_data_bre <- ict_data[format(as.Date(ict_data$OBSERVATION.DATE), "%m") %in% 
                           months, ]

## replace space by "_" in scientific name
ict_data_bre$SCIENTIFIC.NAME <- gsub(" ", "_", ict_data_bre$SCIENTIFIC.NAME)


# separate Sturnella magna into S. neglecta and S. lilianae
## polygons to separate data
smp <- vect("Polygons/sturnella_magna.gpkg")
slp <- vect("Polygons/sturnella_lilianae.gpkg")

## data S. m.
which_sm <- ict_data_bre$SCIENTIFIC.NAME == "Sturnella_magna"

smdata <- ict_data_bre[which_sm, ]
smdata <- vect(as.matrix(smdata[, 4:3]), crs = crs(smp), atts = smdata)

## separate data
smfdata <- as.data.frame(intersect(smdata, smp))
slfdata <- as.data.frame(intersect(smdata, slp))

slfdata$SCIENTIFIC.NAME <- "Sturnella_lilianae"

## merge data
ict_data_bre <- ict_data_bre[!which_sm, ]

ict_data_bre <- rbind(ict_data_bre, smfdata[, -6], slfdata[, -6])


# save the data
save(ict_data_bre, file = "Data/ict_data_breeding.RData")
# ------------------------------------------------------------------------------



# Environmental data -----------------------------------------------------------
# get environmental data
## directory for variables
prism_set_dl_dir("prism_normals")

## list of variables to be downloaded
vars <- c("tmean", "tmin", "tmax", "ppt", "vpdmax", "vpdmin", "tdmean")

## getting variables
var_down <- lapply(vars, function(x) {
  get_prism_normals(type = x, resolution = "4km", mon = 5:7, annual = FALSE, 
                    keepZip = FALSE)
})


# prepare data for future modeling steps
## environmental data
lvar <- pd_to_file(dir("prism_normals"))

all_vars <- raster::stack(lvar)

## perform PCA
pcs <- kuenm_rpca(variables = all_vars, var.scale = TRUE, write.result = TRUE,
                  out.format = "GTiff", out.dir = "PCA_results", n.pcs = 21) 

### PCs to use as variables
pcs <- pcs$PCRasters_initial[[1:4]]
# ------------------------------------------------------------------------------




# Data density treatments ------------------------------------------------------
# do this if using the RData file with filtered records
#load("Data/ict_data_breeding.RData")

# split records by species
ict_data_bre$LONGITUDE <- as.numeric(ict_data_bre$LONGITUDE)
ict_data_bre$LATITUDE <- as.numeric(ict_data_bre$LATITUDE)

ict_by_sp <- split(ict_data_bre, f = as.factor((ict_data_bre$SCIENTIFIC.NAME)))


# filter records so we can have just one per pixel
## finding a variable
file1 <-  pd_to_file("PRISM_vpdmax_30yr_normal_4kmM3_05_bil")

## raster layer from file
varref <- rast(file1)

varref <- project(varref, crs(smp))

## columns with coordinats in occurrence data
xycols <- c("LONGITUDE", "LATITUDE")

## filter one record per pixel, species by species
ict_by_sp_f1 <- lapply(ict_by_sp, function(x) {
  ids <- terra::cellFromXY(varref, x[, xycols])
  x[!duplicated(ids), ]
})


# work only with records related to environmental data
## treatment = all data
ict_by_sp_f1d <- lapply(ict_by_sp_f1, function(x) {
  na.omit(data.frame(x[, c(1, 4:3)], 
                     terra::extract(varref, x[, xycols])[, 1]))[, -4]
})

## treatment = 10% of data
ict_by_sp_f2d <- lapply(ict_by_sp_f1d, function(x) {
  nr <- nrow(x)
  x[sample(nr, size = round(nr / 10)), ]
})

## treatment = 1% of data
ict_by_sp_f3d <- lapply(ict_by_sp_f1d, function(x) {
  nr <- nrow(x)
  x[sample(nr, size = round(nr / 100)), ]
})

## final number of points isn set with fewer records
sapply(ict_by_sp_f3d, nrow)
# ------------------------------------------------------------------------------



# Further filtering -----------------------------------------------------------
# exclude 2 species with not enough points
ict_by_sp_f1d$Icterus_graduacauda <- NULL
ict_by_sp_f1d$Icterus_gularis <- NULL

ict_by_sp_f2d$Icterus_graduacauda <- NULL
ict_by_sp_f2d$Icterus_gularis <- NULL

ict_by_sp_f3d$Icterus_graduacauda <- NULL
ict_by_sp_f3d$Icterus_gularis <- NULL


# restrict records to more realistic areas 
## polygons to restrict records
polys <- list.files("Restriction_polygons", pattern = ".gpkg$", 
                    full.names = TRUE)
polys <- c(NA, polys[1:10], NA, polys[11:17])

polysp <- lapply(polys, function(x) {
  if (is.na(x)) {x} else {vect(x)}
})

## restriction all data
ict_by_sp_f1d <- lapply(1:length(ict_by_sp_f1d), function(x) {
  if (!is.na(polysp[[x]])) {
    sp <- vect(as.matrix(ict_by_sp_f1d[[x]][, 2:3]), crs = crs(polysp[[2]]), 
               atts = ict_by_sp_f1d[[x]])
    
    as.data.frame(intersect(sp, polysp[[x]]))
  } else {
    ict_by_sp_f1d[[x]]
  }
})

## restriction 10% of data
ict_by_sp_f2d <- lapply(1:length(ict_by_sp_f2d), function(x) {
  if (!is.na(polysp[[x]])) {
    sp <- vect(as.matrix(ict_by_sp_f2d[[x]][, 2:3]), crs = crs(polysp[[2]]), 
               atts = ict_by_sp_f2d[[x]])
    
    as.data.frame(intersect(sp, polysp[[x]]))
  } else {
    ict_by_sp_f2d[[x]]
  }
})

## restriction of data
ict_by_sp_f3d <- lapply(1:length(ict_by_sp_f3d), function(x) {
  if (!is.na(polysp[[x]])) {
    sp <- vect(as.matrix(ict_by_sp_f3d[[x]][, 2:3]), crs = crs(polysp[[2]]), 
               atts = ict_by_sp_f3d[[x]])
    
    as.data.frame(intersect(sp, polysp[[x]]))
  } else {
    ict_by_sp_f3d[[x]]
  }
})


# reduced density in E or W of the USA
## reduced to 1% in E
ict_by_sp_fE <- lapply(ict_by_sp_f1d, function(x) {
  e <- x$LONGITUDE >= -101.77
  oe <- x[e, ]
  ne <- nrow(oe)
  oe <- oe[sample(ne, round(ne / 100)), ]
  rbind(oe, x[!e, ])
})

##reduced to 1% in W
ict_by_sp_fW <- lapply(ict_by_sp_f1d, function(x) {
  w <- x$LONGITUDE < -101.77
  ow <- x[w, ]
  nw <- nrow(ow)
  ow <- ow[sample(nw, round(nw / 100)), ]
  rbind(ow, x[!w, ])
})

# save data until this point
save(ict_by_sp_f1d, ict_by_sp_f2d, ict_by_sp_f3d, ict_by_sp_fE, ict_by_sp_fW,
     file = "Data/ict_data_treat_clean.RData")
# ------------------------------------------------------------------------------



# Accessible area simulation ---------------------------------------------------
# exclude species that don't need simulations
sp_exclude <- c("Agelaius_phoeniceus", "Molothrus_ater")

red_data <- ict_by_sp_f1d[!names(ict_by_sp_f1d) %in% sp_exclude]


# thinning data for simualtions
ict_by_sp_f1d_thin <- lapply(red_data, function (x) {
  sp_thin <- thin_data(data = x, longitude = "LONGITUDE", latitude = "LATITUDE", 
                       thin_distance = 50)
})


# M simulation
## species in the list
names(ict_by_sp_f1d_thin)

## variables
lvar <- list.files("prism_normals", pattern = ".bil$", full.names = TRUE, 
                   recursive = TRUE)

all_vars <- raster::stack(lvar)
all_vars <- projectRaster(all_vars, crs = CRS("+init=epsg:4326"))

## directory for results
dirg <- "M_simulations" 

## species names
spnames <- names(ict_by_sp_f1d_thin)

## run in loop
all_ms <- lapply(1:length(spnames), function(w) {
  spname <- spnames[w]
  
  ### SD 1-5, kernel = normal, 125 dispersal events
  sds <- 1:5
  res_folder125 <- paste0(dirg, "/", spname, "_125de_M_normal_SD", sds)
  
  
  M_species_125de <- lapply(1:length(sds), function(x) {
    M_simulationR(data = ict_by_sp_f1d_thin[[spname]],
                  current_variables = all_vars, 
                  dispersal_kernel = "normal", 
                  kernel_spread = sds[x], max_dispersers = 4, 
                  dispersal_events = 125, access_threshold = 5, 
                  out_format = "GTiff", 
                  output_directory = res_folder125[x])
  })
  
  
  ### SD 1-5, kernel = normal, 250 dispersal events
  res_folder250 <- paste0(dirg, "/", spname, "_250de_M_normal_SD", sds)
  
  
  M_species_125de <- lapply(1:length(sds), function(x) {
    M_simulationR(data = ict_by_sp_f1d_thin[[spname]], 
                  current_variables = all_vars, 
                  dispersal_kernel = "normal", 
                  kernel_spread = sds[x], max_dispersers = 4, 
                  dispersal_events = 250, access_threshold = 5, 
                  out_format = "GTiff", 
                  output_directory = res_folder250[x])
  })
})
# ------------------------------------------------------------------------------



# Preparing data for ENM -------------------------------------------------------
# preparing arguments
## species names
spp <- names(ict_by_sp_f1d)

## dispersal events
dev <- c(NA, rep(250, 10), NA, rep(250, 3), 125, rep(250, 3))

## kernel SD
sds <- c(NA, 1, rep(5, 4), 1, 5, 1, 5, 5, NA, rep(5, 3), 3, rep(5, 3))

## directory for models
moddir <- "Modeling"
dir.create(moddir)

## names to write stuff
spfnames <- paste0(moddir, "/", spp, "_all")
spfnames1 <- paste0(moddir, "/", spp, "_10")
spfnames2 <- paste0(moddir, "/", spp, "_1")
spfnames3 <- paste0(moddir, "/", spp, "_E")
spfnames4 <- paste0(moddir, "/", spp, "_W")

## working in loop
premod <- lapply(1:length(spp), function(x) {
  ### variables processing with accessible areas 
  if (!is.na(dev[x])) {
    m <- raster(paste0("M_simulations/", spp[x], "_", dev[x], "de_M_normal_SD", 
                       sds[x], "/accessible_area_M.tif"))
    
    vmas <- mask(crop(pcs, m), m)
  } else {
    vmas <- pcs
  }
  
  ### prepare SWD all data
  dir.create(spfnames[x])
  
  ocnam <- paste0(spfnames[x], "/occ")
  bnam <- paste0(spfnames[x], "/background")
  
  prep <- prepare_swd(occ = ict_by_sp_f1d[[spp[x]]], species = "SCIENTIFIC.NAME", 
                      longitude = "LONGITUDE", latitude = "LATITUDE", 
                      data.split.method = "random", train.proportion = 0.7, 
                      raster.layers = vmas, sample.size = 20000,
                      var.sets = "all_comb", min.number = 2, save = TRUE, 
                      name.occ = ocnam, back.folder = bnam, set.seed = 1)
  
  ### prepare SWD 10%
  dir.create(spfnames1[x])
  
  ocnam <- paste0(spfnames1[x], "/occ")
  bnam <- paste0(spfnames1[x], "/background")
  
  prep <- prepare_swd(occ = ict_by_sp_f2d[[spp[x]]], species = "SCIENTIFIC.NAME", 
                      longitude = "LONGITUDE", latitude = "LATITUDE", 
                      data.split.method = "random", train.proportion = 0.7, 
                      raster.layers = vmas, sample.size = 20000,
                      var.sets = "all_comb", min.number = 2, save = TRUE, 
                      name.occ = ocnam, back.folder = bnam, set.seed = 1)
  
  ### prepare SWD 1%
  dir.create(spfnames2[x])
  
  ocnam <- paste0(spfnames2[x], "/occ")
  bnam <- paste0(spfnames2[x], "/background")
  
  prep <- prepare_swd(occ = ict_by_sp_f3d[[spp[x]]], species = "SCIENTIFIC.NAME", 
                      longitude = "LONGITUDE", latitude = "LATITUDE", 
                      data.split.method = "random", train.proportion = 0.7, 
                      raster.layers = vmas, sample.size = 20000,
                      var.sets = "all_comb", min.number = 2, save = TRUE, 
                      name.occ = ocnam, back.folder = bnam, set.seed = 1)
  
  ### prepare SWD reduced E
  dir.create(spfnames3[x])
  
  ocnam <- paste0(spfnames3[x], "/occ")
  bnam <- paste0(spfnames3[x], "/background")
  
  prep <- prepare_swd(occ = ict_by_sp_fE[[spp[x]]], species = "SCIENTIFIC.NAME", 
                      longitude = "LONGITUDE", latitude = "LATITUDE", 
                      data.split.method = "random", train.proportion = 0.7, 
                      raster.layers = vmas, sample.size = 20000,
                      var.sets = "all_comb", min.number = 2, save = TRUE, 
                      name.occ = ocnam, back.folder = bnam, set.seed = 1)
  
  ### prepare SWD reduced W
  dir.create(spfnames4[x])
  
  ocnam <- paste0(spfnames4[x], "/occ")
  bnam <- paste0(spfnames4[x], "/background")
  
  prep <- prepare_swd(occ = ict_by_sp_fW[[spp[x]]], species = "SCIENTIFIC.NAME", 
                      longitude = "LONGITUDE", latitude = "LATITUDE", 
                      data.split.method = "random", train.proportion = 0.7, 
                      raster.layers = vmas, sample.size = 20000,
                      var.sets = "all_comb", min.number = 2, save = TRUE, 
                      name.occ = ocnam, back.folder = bnam, set.seed = 1)
  
})
# ------------------------------------------------------------------------------



# Ecological niche modeling ----------------------------------------------------
# all folders for models
spp <- list.dirs("Modeling", recursive = FALSE)

# Maxent directory
mx <- "C:/Maxent/3.4.1" # complete path to maxent.jar (change accordingly) ****

# model calibration in loop
mod_cal <- lapply(spp, function(x) {
  sp <- x
  
  ## arguments for model calibration
  oj <- paste0(sp, "/occ_joint.csv") # name of file with all records
  otr <-  paste0(sp, "/occ_train.csv") # name of file with training records
  ote <-  paste0(sp, "/occ_test.csv") # name of file with testing records
  back_dir <- paste0(sp, "/background") # name of directory with backgrounds per set of variables 
  bat <- paste0(sp, "/bash")  # name of file to write Maxent java code
  dir_candidates <- paste0(sp, "/Candidate_models")  # name of directory to write candidate models 
  rgmul <- c(0.1, 0.3, 0.6, 1, 2, 3) # regularization multipliers to test
  fclas <- c("lq", "lp", "lqp") # response types to be used
  sel_type <- "OR_AICc"  # type of selection to be done
  error <- 5  # omission error allowed (corrected threshold)
  dir_eval <- paste0(sp, "/Calibration_results")  # name of directory to write calibration results
  
  ## run calibration (create candidate models and selection of best models)
  cal <- kuenm_cal_swd(occ.joint = oj, occ.tra = otr, occ.test = ote, 
                       back.dir = back_dir, batch = bat, 
                       out.dir.models = dir_candidates, reg.mult = rgmul, 
                       f.clas = fclas, maxent.path = mx, selection = sel_type, 
                       threshold = error, out.dir.eval = dir_eval)
})


# prepare projection layers
## directory for layers
proj_var <- "Modeling/Proj_variables"

if (!dir.exists(proj_var)) {
  dir.create(proj_var) # general directory for projection layers 
}

## sets of variables 
sets <- list.files("Modeling/Agelaius_phoeniceus_1/background/", 
                   full.names = TRUE)
sets1 <- gsub(".csv$", "", 
              list.files("Modeling/Agelaius_phoeniceus_1/background/"))

## preparing projection layer sets in loop
set_prep <- lapply(1:length(sets), function(x) {
  ### sub directories
  setfol <- paste0(proj_var, "/", sets1[x])
  dir.create(setfol) 
  
  curr <- paste0(setfol, "/current")
  dir.create(curr) # calibration scenario
  
  ### vars in set
  vars <- colnames(read.csv(sets[x]))[-(1:3)]
  
  ### write PC raster layers
  pcnames <- paste0(curr, "/", vars, ".asc")
  terra::writeRaster(rast(pcs), filename = pcnames)
})


# final models
mod_fin <- lapply(spp, function(x) {
  sp <- x
  
  ## arguments
  oj <- paste0(sp, "/occ_joint.csv") # name of file with all records
  back_dir <- paste0(sp, "/background") # name of directory with backgrounds per set of variables 
  dir_eval <- paste0(sp, "/Calibration_results")  # name of directory to write calibration results
  bat_mod <- paste0(sp, "/bat_model") # name of file to write Maxent java code 
  reps <- 5 # number of replicates
  rep_type <- "Bootstrap" # type of replicate to be done 
  jack <- TRUE # to get jackknife results 
  out_form <- "cloglog" # Maxent output format
  proj <- TRUE # to project model to other scenarios 
  extrap <- "ext" # type of Maxent extrapolation (free extrapolation)
  mess <- TRUE # to get MESS results
  mod_dir <- paste0(sp, "/Final_models") # name of directory to write final model and projections 
  wait <- TRUE # to wait until Maxent finishes  
  
  ## final model with projections
  kuenm_mod_swd(occ.joint = oj, back.dir = back_dir, out.eval = dir_eval, 
                batch = bat_mod, rep.n = reps, rep.type = rep_type, 
                jackknife = jack, out.format = out_form, project = proj, 
                G.var.dir = proj_var, ext.type = extrap, write.mess = mess, 
                maxent.path = mx, out.dir = mod_dir, wait = wait)
})


# consensus of final models
medians <- lapply(spp, function(x) {
  sp <- x
  
  ## arguments
  spn <- as.character(read.csv(paste0(sp, "/occ_joint.csv"))[1, 1])
  mod_dir <- paste0(sp, "/Final_models") # name of directory with final models
  stat_dir <- paste0(sp, "/Final_Model_Stats") # name of directory to write stats
  
  
  ## final model with projections
  kuenm_modstats_swd(sp.name = spn, fmod.dir = mod_dir, format = "asc", 
                     statistics = c("med", "range"), proj.scenarios = "current", 
                     ext.type = "E", out.dir = stat_dir)
})
# ------------------------------------------------------------------------------



# Preparing data for Zonation --------------------------------------------------
# M layer names
ms <- paste0("M_simulations/", spp, "_", dev, "de_M_normal_SD", 
             sds, "/A_bin.tif")
ms[c(1, 12)] <- NA

## layer to use as a mask to match all layers
hfmask <- rast("Zonation_input/Human_footprint_USA_48_mask_no0.tif")

# all medians of models
spps <- list.dirs("Modeling", recursive = FALSE)
spps <- spps[spps != proj_var]

all_meds <- lapply(spps, function(x) {
  list.files(paste0(x, "/Final_Model_Stats"), pattern = "current_med.tif$", 
             full.names = TRUE, recursive = TRUE)
})

all_meds <- unlist(all_meds)


# preparing data for analysis in loop
## species folders with models
sp1 <- paste0(grep("_1$", spps, value = TRUE), "/occ_joint.csv")
sp10 <- paste0(grep("_10$", spps, value = TRUE), "/occ_joint.csv")
spall <- paste0(grep("_all$", spps, value = TRUE), "/occ_joint.csv")
spE <- paste0(grep("_E$", spps, value = TRUE), "/occ_joint.csv")
spW <- paste0(grep("_W$", spps, value = TRUE), "/occ_joint.csv")

## separating by sets of data
meds1 <- grep("_1/", all_meds, value = TRUE)
meds10 <- grep("_10/", all_meds, value = TRUE)
medsall <- grep("_all/", all_meds, value = TRUE)
medsE <- grep("_E/", all_meds, value = TRUE)
medsW <- grep("_W/", all_meds, value = TRUE)

## destination folders
dir.create("Zonation_projects")

dir1 <- "Zonation_projects/spp_data_1"
dir10 <- "Zonation_projects/spp_data_10"
dirall <- "Zonation_projects/spp_data_all"
dirE <- "Zonation_projects/spp_data_Ered"
dirW <- "Zonation_projects/spp_data_Wred"

dir.create(dir1)
dir.create(dir10)
dir.create(dirall)
dir.create(dirE)
dir.create(dirW)

## running
prep <- lapply(1:length(meds1), function(x) {
  ### read medians of models
  mod1 <- rast(meds1[x])
  mod10 <- rast(meds10[x])
  modall <- rast(medsall[x])
  modE <- rast(medsE[x])
  modW <- rast(medsW[x])
  
  if (!is.na(ms[x])) {
    ### read m layers
    m <- rast(ms[x])

    ### make values outside m = 0
    mod1 <- terra::mask(mod1, m, maskvalue = 0, updatevalue = 0)
    mod10 <- terra::mask(mod10, m, maskvalue = 0, updatevalue = 0)
    modall <- terra::mask(modall, m, maskvalue = 0, updatevalue = 0)
    modE <- terra::mask(modE, m, maskvalue = 0, updatevalue = 0)
    modW <- terra::mask(modW, m, maskvalue = 0, updatevalue = 0)
  }
  
  ### mask to match all layers
  mod1 <- terra::crop(mod1, hfmask, mask = TRUE)
  mod10 <- terra::crop(mod10, hfmask, mask = TRUE)
  modall <- terra::crop(modall, hfmask, mask = TRUE)
  modE <- terra::crop(modE, hfmask, mask = TRUE)
  modW <- terra::crop(modW, hfmask, mask = TRUE)
  
  ### threshold median outputs
  ovals1 <- extract(mod1, as.matrix(read.csv(sp1[x])[, 2:3]))
  ovals10 <- extract(mod10, as.matrix(read.csv(sp10[x])[, 2:3]))
  ovalsall <- extract(modall, as.matrix(read.csv(spall[x])[, 2:3]))
  ovalsE <- extract(modE, as.matrix(read.csv(spE[x])[, 2:3]))
  ovalsW <- extract(modW, as.matrix(read.csv(spW[x])[, 2:3]))
  
  ovals1 <- ovals1[ovals1 > 0]
  ovals10 <- ovals10[ovals10 > 0]
  ovalsall <- ovalsall[ovalsall > 0]
  ovalsE <- ovals10[ovalsE > 0]
  ovalsW <- ovalsall[ovalsW > 0]
  
  thr1 <- ceiling(length(ovals1) * 0.05) + 1
  thr10 <- ceiling(length(ovals10) * 0.05) + 1
  thrall <- ceiling(length(ovalsall) * 0.05) + 1
  thrE <- ceiling(length(ovalsE) * 0.05) + 1
  thrW <- ceiling(length(ovalsW) * 0.05) + 1
  
  modmb1 <- mod1 >= sort(ovals1)[thr1]
  modmb10 <- mod10 >= sort(ovals10)[thr10]
  modmball <- modall >= sort(ovalsall)[thrall]
  modmbE <- modE >= sort(ovalsE)[thrE]
  modmbW <- modW >= sort(ovalsW)[thrW]
  
  ### write them back as GTiff for zonation
  writeRaster(modmb1, filename = paste0(dir1, "/", spp[x], ".tif"))
  writeRaster(modmb10, filename = paste0(dir10, "/", spp[x], ".tif"))
  writeRaster(modmball, filename = paste0(dirall, "/", spp[x], ".tif"))
  writeRaster(modmbE, filename = paste0(dirE, "/", spp[x], ".tif"))
  writeRaster(modmbW, filename = paste0(dirW, "/", spp[x], ".tif"))
})
# ------------------------------------------------------------------------------



# Zonation prioritization ------------------------------------------------------
# path for project
project_path <- "Zonation_projects"

# directory containing the input raster files
input_raster_dir1 <- normalizePath("Zonation_projects/spp_data_1")
input_raster_dir10 <- normalizePath("Zonation_projects/spp_data_10")
input_raster_dirall <- normalizePath("Zonation_projects/spp_data_all")
input_raster_dirE <- normalizePath("Zonation_projects/spp_data_Ered")
input_raster_dirW <- normalizePath("Zonation_projects/spp_data_Wred")

# project name
pr_name1 <- "zonation_data1"
pr_name10 <- "zonation_data10"
pr_nameall <- "zonation_dataall"
pr_nameE <- "zonation_dataEred"
pr_nameW <- "zonation_dataWred"

# variant names 
variant_names <- c("variant_1", "variant_2")

# create projects
prd1 <- create_zproject(name = pr_name1, dir = project_path, 
                        variants = variant_names, 
                        spp_template_dir = input_raster_dir1)

prd10 <- create_zproject(name = pr_name10, dir = project_path, 
                         variants = variant_names, 
                         spp_template_dir = input_raster_dir10)

prdall <- create_zproject(name = pr_nameall, dir = project_path, 
                          variants = variant_names, 
                          spp_template_dir = input_raster_dirall)

prdE <- create_zproject(name = pr_nameE, dir = project_path, 
                        variants = variant_names, 
                        spp_template_dir = input_raster_dirE)

prdW <- create_zproject(name = pr_nameW, dir = project_path, 
                        variants = variant_names, 
                        spp_template_dir = input_raster_dirW)

# second variant runs with protected areas (mask) and human footprint (cost)
## project directories
dpr1 <- "Zonation_projects/zonation_data1"
dpr10 <- "Zonation_projects/zonation_data10"
dprall <- "Zonation_projects/zonation_dataall"
dprE <- "Zonation_projects/zonation_dataEred"
dprW <- "Zonation_projects/zonation_dataWred"

## variant of interest
v1_2 <- readLines(paste0(dpr1, "/variant_2/variant_2.dat"))
v10_2 <- readLines(paste0(dpr10, "/variant_2/variant_2.dat"))
vall_2 <- readLines(paste0(dprall, "/variant_2/variant_2.dat"))
vE_2 <- readLines(paste0(dprE, "/variant_2/variant_2.dat"))
vW_2 <- readLines(paste0(dprW, "/variant_2/variant_2.dat"))

## setting values for mask and cost
maskfile <- normalizePath(
  "Zonation_input/USA_PA_WPA_land_48_ras_mask_12.tif",
  winslash = "/")

costfile <- normalizePath(
  "Zonation_input/Human_footprint_USA_48_mask_no0.tif",
  winslash = "/")

v1_2 <- gsub("^use mask = 0", "use mask = 1", v1_2)
v1_2 <- gsub("^mask file = ", paste0("mask file = ", maskfile), v1_2) 
v1_2 <- gsub("^use cost = 0", "use cost = 1", v1_2)
v1_2 <- gsub("^cost file = ", paste0("cost file = ", costfile), v1_2) 

v10_2 <- gsub("^use mask = 0", "use mask = 1", v10_2)
v10_2 <- gsub("^mask file = ", paste0("mask file = ", maskfile), v10_2) 
v10_2 <- gsub("^use cost = 0", "use cost = 1", v10_2)
v10_2 <- gsub("^cost file = ", paste0("cost file = ", costfile), v10_2)

vall_2 <- gsub("^use mask = 0", "use mask = 1", vall_2)
vall_2 <- gsub("^mask file = ", paste0("mask file = ", maskfile), vall_2) 
vall_2 <- gsub("^use cost = 0", "use cost = 1", vall_2)
vall_2 <- gsub("^cost file = ", paste0("cost file = ", costfile), vall_2)

vE_2 <- gsub("^use mask = 0", "use mask = 1", vE_2)
vE_2 <- gsub("^mask file = ", paste0("mask file = ", maskfile), vE_2) 
vE_2 <- gsub("^use cost = 0", "use cost = 1", vE_2)
vE_2 <- gsub("^cost file = ", paste0("cost file = ", costfile), vE_2)

vW_2 <- gsub("^use mask = 0", "use mask = 1", vW_2)
vW_2 <- gsub("^mask file = ", paste0("mask file = ", maskfile), vW_2) 
vW_2 <- gsub("^use cost = 0", "use cost = 1", vW_2)
vW_2 <- gsub("^cost file = ", paste0("cost file = ", costfile), vW_2)

## save variants back
unlink(paste0(dpr1, "/variant_2/variant_2.dat"))
writeLines(v1_2, paste0(dpr1, "/variant_2/variant_2.dat"))

unlink(paste0(dpr10, "/variant_2/variant_2.dat"))
writeLines(v10_2, paste0(dpr10, "/variant_2/variant_2.dat"))

unlink(paste0(dprall, "/variant_2/variant_2.dat"))
writeLines(vall_2, paste0(dprall, "/variant_2/variant_2.dat"))

unlink(paste0(dprE, "/variant_2/variant_2.dat"))
writeLines(vE_2, paste0(dprE, "/variant_2/variant_2.dat"))

unlink(paste0(dprW, "/variant_2/variant_2.dat"))
writeLines(vW_2, paste0(dprW, "/variant_2/variant_2.dat"))


# get all the bat-files to run zonation
bat_files1 <- normalizePath(list.files(dpr1, pattern = "\\.bat$", 
                                       full.names = TRUE))

bat_files10 <- normalizePath(list.files(dpr10, pattern = "\\.bat$", 
                                        full.names = TRUE))

bat_filesall <- normalizePath(list.files(dprall, pattern = "\\.bat$", 
                                         full.names = TRUE))

bat_filesE <- normalizePath(list.files(dprE, pattern = "\\.bat$", 
                                       full.names = TRUE))

bat_filesW <- normalizePath(list.files(dprW, pattern = "\\.bat$", 
                                       full.names = TRUE))

# run all the bat files
for (i in 1:length(bat_files1)) {
  run_bat(bat_files1[i], exe = "zig4")
  run_bat(bat_files10[i], exe = "zig4")
  run_bat(bat_filesall[i], exe = "zig4")
  run_bat(bat_filesE[i], exe = "zig4")
  run_bat(bat_filesW[i], exe = "zig4")
}
# ------------------------------------------------------------------------------



# Processing Zonation results --------------------------------------------------
# folder for results
dir.create("Results_zonation")

# load project results
prd1 <- load_zproject(dpr1)
prd10 <- load_zproject(dpr10)
prdall <- load_zproject(dprall)
prdE <- load_zproject(dprE)
prdW <- load_zproject(dprW)

## get variants
v1_1 <- get_variant(prd1, 1)
v10_1 <- get_variant(prd10, 1)
vall_1 <- get_variant(prdall, 1)
vE_1 <- get_variant(prdE, 1)
vW_1 <- get_variant(prdW, 1)

v1_2 <- get_variant(prd1, 2)
v10_2 <- get_variant(prd10, 2)
vall_2 <- get_variant(prdall, 2)
vE_2 <- get_variant(prdE, 2)
vW_2 <- get_variant(prdW, 2)

## get results for all variants
rv1_1 <- results(v1_1)
rv10_1 <- results(v10_1)
rvall_1 <- results(vall_1)
rvE_1 <- results(vE_1)
rvW_1 <- results(vW_1)

rv1_2 <- results(v1_2)
rv10_2 <- results(v10_2)
rvall_2 <- results(vall_2)
rvE_2 <- results(vE_2)
rvW_2 <- results(vW_2)

## land proportion and distribution proportion curves
curv1_1 <- as.data.frame(curves(rv1_1))
curv10_1 <- as.data.frame(curves(rv10_1))
curvall_1 <- as.data.frame(curves(rvall_1))
curvE_1 <- as.data.frame(curves(rvE_1))
curvW_1 <- as.data.frame(curves(rvW_1))

curv1_2 <- as.data.frame(curves(rv1_2))
curv10_2 <- as.data.frame(curves(rv10_2))
curvall_2 <- as.data.frame(curves(rvall_2))
curvE_2 <- as.data.frame(curves(rvE_2))
curvW_2 <- as.data.frame(curves(rvW_2))

## save data to plot cuerves
write.csv(curv1_1, file = "Results_zonation/Zonation_curves_1_1.csv", 
          row.names = FALSE)
write.csv(curv10_1, file = "Results_zonation/Zonation_curves_10_1.csv", 
          row.names = FALSE)
write.csv(curvall_1, file = "Results_zonation/Zonation_curves_all_1.csv", 
          row.names = FALSE)
write.csv(curvE_1, file = "Results_zonation/Zonation_curves_Ered_1.csv", 
          row.names = FALSE)
write.csv(curvW_1, file = "Results_zonation/Zonation_curves_Wred_1.csv", 
          row.names = FALSE)

write.csv(curv1_2, file = "Results_zonation/Zonation_curves_1_2.csv", 
          row.names = FALSE)
write.csv(curv10_2, file = "Results_zonation/Zonation_curves_10_2.csv", 
          row.names = FALSE)
write.csv(curvall_2, file = "Results_zonation/Zonation_curves_all_2.csv", 
          row.names = FALSE)
write.csv(curvE_2, file = "Results_zonation/Zonation_curves_Ered_2.csv", 
          row.names = FALSE)
write.csv(curvW_2, file = "Results_zonation/Zonation_curves_Wred_2.csv", 
          row.names = FALSE)


# species richness for comparison with results
spp1 <- rast(list.files(input_raster_dir1, pattern = ".tif$", full.names = T))
spp1 <- app(spp1, sum, filename = "Results_zonation/spp_richness_1.tif")

spp10 <- rast(list.files(input_raster_dir10, pattern = ".tif$", full.names = T))
spp10 <- app(spp10, sum, filename = "Results_zonation/spp_richness_10.tif")

sppall <- rast(list.files(input_raster_dirall, pattern = ".tif$", full.names = T))
sppall <- app(sppall, sum, filename = "Results_zonation/spp_richness_all.tif")

sppE <- rast(list.files(input_raster_dirE, pattern = ".tif$", full.names = T))
sppE <- app(sppE, sum, filename = "Results_zonation/spp_richness_Ered.tif")

sppW <- rast(list.files(input_raster_dirW, pattern = ".tif$", full.names = T))
sppW <- app(sppW, sum, filename = "Results_zonation/spp_richness_Wred.tif")


# analyses to show where results differ the most
density3_1 <- paste0(c(dpr1, dpr10, dprall), 
                     "/variant_1/variant_1_out/variant_1.rank.compressed.tif")
varmap_1 <- app(rast(density3_1), var, 
                filename = "Results_zonation/variance_1_10_all_1.tif")

density3_2 <- paste0(c(dpr1, dpr10, dprall), 
                     "/variant_2/variant_2_out/variant_2.rank.compressed.tif")
varmap_2 <- app(rast(density3_2), var, 
                filename = "Results_zonation/variance_1_10_all_2.tif")

densityEW_1 <- paste0(c(dprE, dprW), 
                      "/variant_1/variant_1_out/variant_1.rank.compressed.tif")
diff_1 <- app(rast(densityEW_1), diff)

diff_1 <- app(diff_1, abs, filename = "Results_zonation/difference_E_W_1.tif")


densityEW_2 <- paste0(c(dprE, dprW), 
                      "/variant_2/variant_2_out/variant_2.rank.compressed.tif")
diff_2 <- app(rast(densityEW_2), diff)

diff_2 <- app(diff_2, abs, filename = "Results_zonation/difference_E_W_2.tif")


# defining threshold for priority maps
## finding critical value via minimum protected range
curv1_1_min <- apply(curv1_1[, 8:26], 1, min)
curv10_1_min <- apply(curv10_1[, 8:26], 1, min)
curvall_1_min <-apply(curvall_1[, 8:26], 1, min)
curvE_1_min <- apply(curvE_1[, 8:26], 1, min)
curvW_1_min <- apply(curvW_1[, 8:26], 1, min)

curv1_2_min <- apply(curv1_2[, 8:26], 1, min)
curv10_2_min <- apply(curv10_2[, 8:26], 1, min)
curvall_2_min <-apply(curvall_2[, 8:26], 1, min)
curvE_2_min <- apply(curvE_2[, 8:26], 1, min)
curvW_2_min <- apply(curvW_2[, 8:26], 1, min)

## construct the table
pr_thres <- data.frame(
  Treatment = c(
    paste0("Density_", c(1, 10, 100), "_precent"), 
    paste0("Desity_reduced_", c("E", "W")),
    paste0("Density_", c(1, 10, 100), "_precent_cost"), 
    paste0("Desity_reduced_", c("E", "W"), "_cost")),
  Threshold_1_percent = c(
    curv1_1[curv1_1_min == 0.01, "pr_lost"][1],
    curv10_1[curv10_1_min == 0.01, "pr_lost"][1],
    curvall_1[curvall_1_min == 0.01, "pr_lost"][1],
    curvE_1[curvE_1_min == 0.01, "pr_lost"][1],
    curvW_1[curvW_1_min == 0.01, "pr_lost"][1],
    curv1_2[curv1_2_min == 0.01, "pr_lost"][1],
    curv10_2[curv10_2_min == 0.01, "pr_lost"][1],
    curvall_2[curvall_2_min == 0.01, "pr_lost"][1],
    curvE_2[curvE_2_min == 0.01, "pr_lost"][1],
    curvW_2[curvW_2_min == 0.01, "pr_lost"][1]
  ),
  Threshold_5_percent = c(
    curv1_1[curv1_1_min == 0.05, "pr_lost"][1],
    curv10_1[curv10_1_min == 0.05, "pr_lost"][1],
    curvall_1[curvall_1_min == 0.05, "pr_lost"][1],
    curvE_1[curvE_1_min == 0.05, "pr_lost"][1],
    curvW_1[curvW_1_min == 0.05, "pr_lost"][1],
    curv1_2[curv1_2_min == 0.05, "pr_lost"][1],
    curv10_2[curv10_2_min == 0.05, "pr_lost"][1],
    curvall_2[curvall_2_min == 0.05, "pr_lost"][1],
    curvE_2[curvE_2_min == 0.05, "pr_lost"][1],
    curvW_2[curvW_2_min == 0.05, "pr_lost"][1]
  ),
  Threshold_10_percent = c(
    curv1_1[curv1_1_min == 0.1, "pr_lost"][1],
    curv10_1[curv10_1_min == 0.1, "pr_lost"][1],
    curvall_1[curvall_1_min == 0.1, "pr_lost"][1],
    curvE_1[curvE_1_min == 0.1, "pr_lost"][1],
    curvW_1[curvW_1_min == 0.1, "pr_lost"][1],
    curv1_2[curv1_2_min == 0.1, "pr_lost"][1],
    curv10_2[curv10_2_min == 0.1, "pr_lost"][1],
    curvall_2[curvall_2_min == 0.1, "pr_lost"][1],
    curvE_2[curvE_2_min == 0.1, "pr_lost"][1],
    curvW_2[curvW_2_min == 0.1, "pr_lost"][1]
  )
)

## write results
write.csv(pr_thres, file = "Results_zonation/Threshold_zonation_results.csv", 
          row.names = FALSE)


# threshold all zonation maps
## all zonation priority rastes
zres <- rast(c(density3_1, densityEW_1, density3_2, densityEW_2))

## threshold
zres_t1 <- lapply(1:nlyr(zres), function(x) {
  zres[[x]] >= pr_thres$Threshold_1_percent[x]
})

zres_t5 <- lapply(1:nlyr(zres), function(x) {
  zres[[x]] >= pr_thres$Threshold_5_percent[x]
})

zres_t10 <- lapply(1:nlyr(zres), function(x) {
  zres[[x]] >= pr_thres$Threshold_10_percent[x]
})

## names for results
zt1_names <- paste0("Results_zonation/", pr_thres$Treatment, "_thr1.tif")
zt5_names <- paste0("Results_zonation/", pr_thres$Treatment, "_thr5.tif")
zt10_names <- paste0("Results_zonation/", pr_thres$Treatment, "_thr10.tif")

## write results
writeRaster(rast(zres_t1), filename = zt1_names)
writeRaster(rast(zres_t5), filename = zt5_names)
writeRaster(rast(zres_t10), filename = zt10_names)
# ------------------------------------------------------------------------------